/*

MetaTrader History Toolkit
example.cpp
v1.1
2018-08-28
Copyright  2013-2018 Heinz Traub

*/

#include <windows.h>
#include <cstdio>
#include <ctime>
#include <memory>
#include <string>
#include "hcc.hpp"

void pause();
bool CheckHCCError(ReturnCode);

int main(int argc, char* argv[])
{
	printf("MetaTrader History Toolkit - C++11 Example\n");
	printf("Copyright (C) 2013-2018 Heinz Traub\n");
	if(argc > 1)
	{
		int nChars = MultiByteToWideChar(CP_UTF8, 0, argv[1], -1, NULL, 0);
		if(nChars > 0)
		{
			std::unique_ptr<wchar_t[]> hcc_path = std::make_unique<wchar_t[]>(nChars);
			if(MultiByteToWideChar(CP_UTF8, 0, argv[1], -1, hcc_path.get(), nChars) == nChars)
			{
				HCCHandle file;
				ReturnCode rc = HCCOpenFile(hcc_path.get(), &file);
				if(!CheckHCCError(rc))
					return 1;
				wprintf(L"\nFile: %ls\n", hcc_path.get());
				
				wchar_t wsymbol[10]; // Should fit any symbol.
				rc = HCCGetSymbol(file, wsymbol);
				if(!CheckHCCError(rc))
					return 1;
				wprintf(L"Symbol: %ls\n", wsymbol);
				
				printf("Timeframe: 1 Minute\n");
				
				printf("Loading history...");
				unsigned int hlength;
				rc = HCCLoadHistory(file, static_cast<unsigned int>(HCC_TIMEFRAME::M1), 0, 3, &hlength); // Load only the first 3 rates.
				if(!CheckHCCError(rc))
					return 1;
				printf("\nRates Loaded: %u", hlength);
				if(hlength > 0)
				{
					printf("\nThis program will display the first 3 rates (if available):\n");
					pause();
					HCCRates r;
					unsigned int precision;
					rc = HCCGetPrecision(file, &precision);
					if(!CheckHCCError(rc))
						return 1;
					char price_format[10];
					sprintf(price_format, "\n%%s: %%.%uf", precision);
					
					for(unsigned int i = 0; i < hlength; i++)
					{
						rc = HCCGetRates(file, i, &r);
						if(!CheckHCCError(rc))
							return 1;
						
						printf("\n\n[RATES #%u]", i + 1);
						time_t tt = r.time;
						tm rtime = *gmtime(&tt);
						printf("\nTime: %04u.%02u.%02u %02u:%02u", rtime.tm_year + 1900, rtime.tm_mon + 1, rtime.tm_mday, rtime.tm_hour, rtime.tm_min);
						printf(price_format, "Open", r.open);
						printf(price_format, "High", r.high);
						printf(price_format, "Low", r.low);
						printf(price_format, "Close", r.close);
						printf("\nVolume: %u", r.volume);
						printf("\nSpread: %u", r.spread);
					}
					
					rc = HCCCloseFile(file);
					if(!CheckHCCError(rc))
						return 1;
					printf("\n\nDone!\n\n");
				}
				else
					printf("\nNo rates found, halting.\n");
			}
			else
			{
				printf("Error: Couldn't map widechars.\n");
				pause();
				return 1;
			}
		}
		else
		{
			printf("Error: Couldn't allocate buffer for path.\n");
			pause();
			return 1;
		}
	}
	else // Show usage.
		printf("\nUsage: Drag and drop a valid HCC file to example.exe in Windows shell.\n\n");
	pause();
	return 0;
}

void pause()
{
	system("pause");
}

bool CheckHCCError(ReturnCode rc)
{
	if(rc != HCC_SUCCESS)
	{
		wchar_t wmessage[128]; // Should fit any error message.
		std::wstring werror = L"HCC Error (%u)";
		if(HCCGetErrorMessage(rc, 0, wmessage) == HCC_SUCCESS)
		{
			werror += L": ";
			werror += wmessage;
		}
		wprintf(L"\n%ls\n", werror.c_str());
		pause();
		return false;
	}
	else
		return true;
}