﻿/*

MetaTrader History Toolkit
example.d
v1.1
2018-08-28
Copyright © 2013-2018 Heinz Traub

*/

// Compatible with D 2.0 specs.

private import hcc, core.stdc.stdlib, core.stdc.time, std.stdio, std.utf, std.string, std.conv;

extern(C) void* gc_getProxy();

int main(string[] args)
{
	writeln("MetaTrader History Toolkit - D Example");
	writeln("Copyright (C) 2013-2018 Heinz Traub");
	if(args.length > 1)
	{
		HCCInitialize(gc_getProxy());
		HCCHandle file;
		ReturnCode rc = HCCOpenFile(toUTF16z(args[1]), file);
		if(!CheckHCCError(rc))
			return 1;
		writefln("\nFile: %s", args[1]);
		
		wchar[10] wsymbol; // Should fit any symbol.
		rc = HCCGetSymbol(file, wsymbol.ptr);
		if(!CheckHCCError(rc))
			return 1;
		writefln("Symbol: %s", to!string(wsymbol.ptr));
		
		writefln("Timeframe: 1 Minute");
		
		write("Loading history...");
		uint hlength;
		rc = HCCLoadHistory(file, HCC_TIMEFRAME.M1, 0, 3, hlength); // Load only the first 3 rates.
		if(!CheckHCCError(rc))
			return 1;
		writefln("\nRates Loaded: %u", hlength);
		if(hlength > 0)
		{
			writeln("This program will display the first 3 rates (if available):");
			pause();
			HCCRates r;
			uint precision;
			rc = HCCGetPrecision(file, precision);
			if(!CheckHCCError(rc))
				return 1;
			string price_format = "%." ~ to!string(precision) ~ "f";
			
			for(uint i = 0; i < hlength; i++)
			{
				rc = HCCGetRates(file, i, r);
				if(!CheckHCCError(rc))
					return 1;
				
				writefln("\n[RATES #%u]", i + 1);
				tm rtime = *gmtime(cast(time_t*)&r.time);
				writefln("Time: %04u.%02u.%02u %02u:%02u", rtime.tm_year + 1900, rtime.tm_mon + 1, rtime.tm_mday, rtime.tm_hour, rtime.tm_min);
				writefln("Open: " ~ price_format, r.open);
				writefln("High: " ~ price_format, r.high);
				writefln("Low: " ~ price_format, r.low);
				writefln("Close: " ~ price_format, r.close);
				writefln("Volume: %u", r.volume);
				writefln("Spread: %u", r.spread);
			}
			
			rc = HCCCloseFile(file);
			if(!CheckHCCError(rc))
				return 1;
			writeln("\nDone!\n");
		}
		else
			writeln("No rates found, halting.");
		HCCTerminate();
	}
	else // Show usage.
		writeln("\nUsage: Drag and drop a valid HCC file to example.exe in Windows shell.\n");
	pause();
	return 0;
}

void pause()
{
	system(toStringz("pause"));
}

bool CheckHCCError(ReturnCode rc)
{
	if(rc != HCC_SUCCESS)
	{
		wchar[128] wmessage; // Should fit any error message.
		string err_str = "HCC Error (%u)";
		if(HCCGetErrorMessage(rc, 0, wmessage.ptr) == HCC_SUCCESS)
			err_str ~= ": " ~ to!string(wmessage.ptr);
		writefln(err_str, rc);
		pause();
		return false;
	}
	else
		return true;
}