﻿/*

MetaTrader History Toolkit
hcc.di
v1.1
2018-08-28
Copyright © 2012-2018 Heinz Traub

*/

// HCC bindings for the D programming language spec 2.

module hcc;

alias uint ReturnCode;
alias uint HCCHandle;

const ReturnCode HCC_SUCCESS = 0;

enum HCC_RATES_PROPERTY : int
{
	Time,
	Open,
	High,
	Low,
	Close,
	Volume,
	Spread
}

enum HCC_TIMEFRAME : int
{
	M1 = 1, // 1 Minute.
	M2 = 2, // 2 Minutes.
	M3 = 3, // 3 Minutes.
	M4 = 4, // 4 Minutes.
	M5 = 5, // 5 Minutes.
	M6 = 6, // 6 Minutes.
	M10 = 10, // 10 Minutes.
	M12 = 12, // 12 Minutes.
	M15 = 15, // 15 Minutes.
	M20 = 20, // 20 Minutes.
	M30 = 30, // 30 Minutes.
	H1 = 60, // 1 Hour.
	H2 = 120, // 2 Hours.
	H3 = 180, // 3 Hours.
	H4 = 240, // 4 Hours.
	H6 = 360, // 6 Hours.
	H8 = 480, // 8 Hours.
	H12 = 720, // 12 Hours.
	D1 = 1440, // 1 Day.
	W1 = 10080, // 1 Week.
	MN28 = 40320, // 1 Month containing 28 days.
	MN29 = 41760, // 1 Month containing 29 days.
	MN30 = 43200, // 1 Month containing 30 days.
	MN31 = 44640, // 1 Month containing 31 days.
	MN = MN30, // Standard Month.
	TM = MN * 3, // 1 Trimester.
	SM = MN * 6, // 1 Semester.
	Y1 = 525600, // 1 Year.
	LY1 = 527040 // 1 Leap year.
}

enum HCC_RATES_SEPARATOR : int
{
	None = 0,
	Standard = 1,
	Custom = 0x7FFFFFFF
}

struct HCCRates
{
	align(1): // No alignment is performed. Members are packed together to match the behaviour in MQL5.
	uint time;
	double open;
	double high;
	double low;
	double close;
	uint volume;
	uint spread;
}

struct HCCParsingOptions
{
	align(1): // No alignment is performed. Members are packed together to match the behaviour in MQL5.
	HCC_RATES_SEPARATOR separator;
	ubyte[8] custom_separator;
	ubyte custom_length;
	bool fast_parse;
	bool halt_corrupt;
	bool ignore_corrupt;
}

extern(C)
{
	void HCCInitialize(void* gc);
	void HCCTerminate();
	ReturnCode HCCGetErrorMessage(ReturnCode code, ushort language, wchar* message);
	ReturnCode HCCOpenFile(const(wchar)* path, out HCCHandle file);
	ReturnCode HCCCloseFile(HCCHandle file);
	ReturnCode HCCGetSymbol(HCCHandle file, wchar* symbol);
	ReturnCode HCCGetPrecision(HCCHandle file, out uint precision);
	ReturnCode HCCSetParsingOptions(HCCHandle file, HCCParsingOptions options);
	ReturnCode HCCLoadHistory(HCCHandle file, uint period, uint start, uint how_many, out uint load_count);
	ReturnCode HCCUnloadHistory(HCCHandle file);
	ReturnCode HCCGetHistoryLength(HCCHandle file, out uint length);
	ReturnCode HCCGetHistoryPeriod(HCCHandle file, out uint period);
	ReturnCode HCCGetRates(HCCHandle file, uint index, out HCCRates rates);
	ReturnCode HCCGetRatesDouble(HCCHandle file, uint index, HCC_RATES_PROPERTY prop, out double value);
	ReturnCode HCCGetRatesInteger(HCCHandle file, uint index, HCC_RATES_PROPERTY prop, out uint value);
	ReturnCode HCCBeginTransaction(HCCHandle file);
	ReturnCode HCCCommitTransaction(HCCHandle file, out uint updated_count);
	ReturnCode HCCUpdateRates(HCCHandle file, uint index, HCCRates rates);
	ReturnCode HCCExportHistoryCSV(HCCHandle file, const(wchar)* csv_path, const(wchar)* sep, const(wchar)* decim, ushort language, uint start, uint how_many, out uint export_count);
	ReturnCode HCCExportCSV(HCCHandle file, const(wchar)* csv_path, const(wchar)* sep, const(wchar)* decim, ushort language, uint period, uint start, uint how_many, out uint export_count);
	ReturnCode HCC2CSV(const(wchar)* hcc_path, const(wchar)* csv_path, const(wchar)* sep, const(wchar)* decim, ushort language, uint period, uint start, uint how_many, out uint export_count);
}