/*

MetaTrader History Toolkit
hcc.h
v1.1
2018-08-28
Copyright  2012-2018 Heinz Traub

*/

// HCC bindings for C and legacy C++ (up to C++03).

#include <stdbool.h>

typedef unsigned int ReturnCode;
typedef unsigned int HCCHandle;

const ReturnCode HCC_SUCCESS = 0;

enum HCC_RATES_PROPERTY
{
	Time,
	Open,
	High,
	Low,
	Close,
	Volume,
	Spread
};

enum HCC_TIMEFRAME
{
	M1 = 1, // 1 Minute.
	M2 = 2, // 2 Minutes.
	M3 = 3, // 3 Minutes.
	M4 = 4, // 4 Minutes.
	M5 = 5, // 5 Minutes.
	M6 = 6, // 6 Minutes.
	M10 = 10, // 10 Minutes.
	M12 = 12, // 12 Minutes.
	M15 = 15, // 15 Minutes.
	M20 = 20, // 20 Minutes.
	M30 = 30, // 30 Minutes.
	H1 = 60, // 1 Hour.
	H2 = 120, // 2 Hours.
	H3 = 180, // 3 Hours.
	H4 = 240, // 4 Hours.
	H6 = 360, // 6 Hours.
	H8 = 480, // 8 Hours.
	H12 = 720, // 12 Hours.
	D1 = 1440, // 1 Day.
	W1 = 10080, // 1 Week.
	MN28 = 40320, // 1 Month containing 28 days.
	MN29 = 41760, // 1 Month containing 29 days.
	MN30 = 43200, // 1 Month containing 30 days.
	MN31 = 44640, // 1 Month containing 31 days.
	MN = MN30, // Standard Month.
	TM = MN * 3, // 1 Trimester.
	SM = MN * 6, // 1 Semester.
	Y1 = 525600, // 1 Year.
	LY1 = 527040 // 1 Leap year.
};

enum HCC_RATES_SEPARATOR
{
	None = 0,
	Standard = 1,
	Custom = 0x7FFFFFFF
};

#pragma pack(1) // No alignment is performed. Members are packed together to match the behaviour in MQL5.

struct HCCRates
{
	unsigned int time;
	double open;
	double high;
	double low;
	double close;
	unsigned int volume;
	unsigned int spread;
};

struct HCCParsingOptions
{
	enum HCC_RATES_SEPARATOR separator;
	unsigned char custom_separator[8];
	unsigned char custom_length;
	bool fast_parse;
	bool halt_corrupt;
	bool ignore_corrupt;
};

#ifdef __cplusplus
extern "C"
{
#endif
	__declspec(dllimport) ReturnCode HCCGetErrorMessage(ReturnCode code, unsigned short int language, wchar_t* message);
	__declspec(dllimport) ReturnCode HCCOpenFile(const wchar_t* path, HCCHandle* file);
	__declspec(dllimport) ReturnCode HCCCloseFile(HCCHandle file);
	__declspec(dllimport) ReturnCode HCCGetSymbol(HCCHandle file, wchar_t* symbol);
	__declspec(dllimport) ReturnCode HCCGetPrecision(HCCHandle file, unsigned int* precision);
	__declspec(dllimport) ReturnCode HCCSetParsingOptions(HCCHandle file, struct HCCParsingOptions options);
	__declspec(dllimport) ReturnCode HCCLoadHistory(HCCHandle file, unsigned int period, unsigned int start, unsigned int how_many, unsigned int* load_count);
	__declspec(dllimport) ReturnCode HCCUnloadHistory(HCCHandle file);
	__declspec(dllimport) ReturnCode HCCGetHistoryLength(HCCHandle file, unsigned int* length);
	__declspec(dllimport) ReturnCode HCCGetHistoryPeriod(HCCHandle file, unsigned int* period);
	__declspec(dllimport) ReturnCode HCCGetRates(HCCHandle file, unsigned int index, struct HCCRates* rates);
	__declspec(dllimport) ReturnCode HCCGetRatesDouble(HCCHandle file, unsigned int index, enum HCC_RATES_PROPERTY prop, double* value);
	__declspec(dllimport) ReturnCode HCCGetRatesInteger(HCCHandle file, unsigned int index, enum HCC_RATES_PROPERTY prop, unsigned int* value);
	__declspec(dllimport) ReturnCode HCCBeginTransaction(HCCHandle file);
	__declspec(dllimport) ReturnCode HCCCommitTransaction(HCCHandle file, unsigned int* updated_count);
	__declspec(dllimport) ReturnCode HCCUpdateRates(HCCHandle file, unsigned int index, struct HCCRates rates);
	__declspec(dllimport) ReturnCode HCCExportHistoryCSV(HCCHandle file, const wchar_t* csv_path, const wchar_t* sep, const wchar_t* decim, unsigned short int language, unsigned int start, unsigned int how_many, unsigned int* export_count);
	__declspec(dllimport) ReturnCode HCCExportCSV(HCCHandle file, const wchar_t* csv_path, const wchar_t* sep, const wchar_t* decim, unsigned short int language, unsigned int period, unsigned int start, unsigned int how_many, unsigned int* export_count);
	__declspec(dllimport) ReturnCode HCC2CSV(const wchar_t* hcc_path, const wchar_t* csv_path, const wchar_t* sep, const wchar_t* decim, unsigned short int language, unsigned int period, unsigned int start, unsigned int how_many, unsigned int* export_count);
#ifdef __cplusplus
}
#endif